﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using CoffeeStoreAddin.VisExtensions;
using Visio = Microsoft.Office.Interop.Visio;

namespace CoffeeStoreAddin
{
    public class FurnitureManager
    {
        const string TableMstNameU = "Table";
        const string ChairMstNameU = "Chair";
        const string ChairAngleCellName = "Controls.ChairAngle";
        const string StencilFileName = "CoffeeStore.vssx";
        const double PI = Math.PI;

        internal void AddChair(Visio.Shape hostShape)
        {
            if(hostShape == null)
                throw new ArgumentNullException(nameof(hostShape));

            var chairsSelection = GetNeighborShapes(hostShape, ChairMstNameU);
            var vPag = hostShape.ContainingPage;
            var drawingWdw = hostShape.GetDrawingWindows()?.FirstOrDefault();
            var mstChair = drawingWdw?.DockedStencil(StencilFileName)?.Masters?[ChairMstNameU];
            if (mstChair != null)
            {
                var newChair = vPag.Drop(mstChair,
                                         hostShape.CellsU["PinX"].ResultIU,
                                         hostShape.CellsU["PinY"].ResultIU);
                if (chairsSelection.Count > 0)
                {
                    newChair.CellsU["Width"].FormulaU = chairsSelection[1].CellsU["Width"].FormulaU;
                    newChair.CellsU["Height"].FormulaU = chairsSelection[1].CellsU["Height"].FormulaU;
                }
                chairsSelection.Select(newChair, (short)Visio.VisSelectArgs.visSelect);

                ArrangeRadial(hostShape, chairsSelection); 
            }
        }


        private bool IsArrangeable(Visio.Shape shpIn)
        {
            return shpIn.IsMasterInstanceOf(new string [] { TableMstNameU, ChairMstNameU });
        }


        internal void SelectAllChairs(Visio.Shape shp)
        {
            var chairsSelection = GetNeighborShapes(shp, ChairMstNameU);
            var wdw = shp.GetDrawingWindows()?.FirstOrDefault();
            if (wdw != null)
            {
                wdw.DeselectAll();
                wdw.Selection = chairsSelection;
            }
        }


        internal Visio.Selection GetNeighborShapes(Visio.Shape targetShp, string mstNameU)
        {
            Visio.Selection sel;
            try
            {
                sel = targetShp.SpatialNeighbors[(short)((int)Visio.VisSpatialRelationCodes.visSpatialOverlap
                                                                  + (int)Visio.VisSpatialRelationCodes.visSpatialContain),
                                                                  0,
                                                                  (short)Visio.VisSpatialRelationFlags.visSpatialBackToFront];
                //Filter selection for shapes based on the incoming master name
                for (var i = sel.Count; i >= 1; i--)
                {
                    if (!sel[i].IsMasterInstanceOf(mstNameU))
                    {
                        sel.Select(sel[i], (short)Visio.VisSelectArgs.visDeselect);
                    }
                }
            }
            catch (Exception ex)
            {   
                Debug.WriteLine(ex.Message);
                throw;
            }
            return sel;
        }


        private static void ArrangeRadial(Visio.Shape centerShp, Visio.Selection orbitSel)
        {
            var cX = centerShp.CellsU["PinX"].ResultIU;
            var cY = centerShp.CellsU["PinY"].ResultIU;
            var cShpRadius = centerShp.CellsU["Width"].ResultIU/2;

            var angleInc = (double)360/orbitSel.Count;

            try
            {
                for (int i = 1; i <= orbitSel.Count; i++)
                {
                    var radians = (angleInc * i) * PI / 180;
                    var itemRadius = cShpRadius + (orbitSel[i].CellsU["Width"].ResultIU / 2) * 0.8;
                    orbitSel[i].CellsU["PinX"].FormulaU = $"{centerShp.NameID}!PinX+{itemRadius * Math.Cos(radians)}";
                    orbitSel[i].CellsU["PinY"].FormulaU = $"{centerShp.NameID}!PinY+{itemRadius * Math.Sin(radians)}";
                    //orientate handle
                    if (orbitSel[i].CellExistsU[ChairAngleCellName, (short)Visio.VisExistsFlags.visExistsAnywhere] != 0)
                    {
                        double localX;
                        double localY;
                        orbitSel[i].XYFromPage(cX, cY, out localX, out localY);
                        orbitSel[i].CellsU[ChairAngleCellName].FormulaU =
                            $"Width*{localX / orbitSel[i].CellsU["Width"].ResultIU}";
                        orbitSel[i].CellsU[$"{ChairAngleCellName}.Y"].FormulaU =
                            $"Height*{localY / orbitSel[i].CellsU["Height"].ResultIU}";
                    }
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
                throw;
            }
        }


        private void ArrangeTable(Visio.Shape shpTable)
        {
            var chairsSelection = GetNeighborShapes(shpTable, ChairMstNameU);
            if (chairsSelection.Count > 0)
                ArrangeRadial(shpTable, chairsSelection);
        }


        public Queue<Visio.Shape> TablesForArrange { get; set; }

        internal void FlagTablesForArrange(Visio.Selection selIn)
        {
            foreach (Visio.Shape shp in selIn)
            {
                FlagTablesForArrange(shp);
            }
        }


        internal void FlagTablesForArrange(Visio.Shape shpIn)
        {
            if (IsArrangeable(shpIn))
            {
                if (TablesForArrange == null)
                    TablesForArrange = new Queue<Visio.Shape>();

                var sel = GetNeighborShapes(shpIn, TableMstNameU);
                if (sel?.Count > 0 && !TablesForArrange.Contains(sel[1]))
                {
                    TablesForArrange.Enqueue(sel[1]);
                } 
            }
        }


        internal void ProcessFlaggedTables()
        {
            if (TablesForArrange == null) return;

            while (TablesForArrange.Count > 0)
            {
                ArrangeTable(TablesForArrange.Dequeue());
            }
        }
    }
}
