﻿
class VisioTemplateDraw
{
    #[Microsoft.Office.Interop.Visio.Application] $vApp
    $vApp
    $vDoc
    $vCurrentPage
    $vCurrentParentShape
    [bool] $errorLoadingTemplate
    [System.Collections.Hashtable] $masters = @{}
    [object[]] $stencilWindows

    

    hidden [string] $stencilMasterDelimiter = ';'


    hidden [void] Init($templateName)
    {
        try
        {
            $this.vApp = New-Object -ComObject 'Visio.Application'
            $this.vDoc = $this.vApp.Documents.Add($templateName)
            if ($this.errorLoadingTemplate -ne $true)
            {
                $this.vCurrentPage = $this.vDoc.Pages[1]
                $this.RefreshLocals($this.vDoc.Name)
            }
        }
        catch [System.Runtime.InteropServices.COMException]
        {
            if ($_.Exception.ErrorCode -eq -2032465466)
            {
                $this.errorLoadingTemplate = $true
                $line = $_.InvocationInfo.ScriptLineNumber
                Write-Host "Visio was unable to find template named '$templateName' `(File not found - Line $line`)" -ForegroundColor Red
            }          
        }
    }


    hidden [void] RefreshLocals([string] $drawingDocName)
    {
        #Reset masters hashtable and repopulate stencil docs array
        $this.masters = @{}
        $this.stencilWindows = ($this.vApp.Windows | Where-Object {$_.Caption -eq $drawingDocName}).Windows | Where-Object {$_.type -eq 7 -and $_.Caption -ne ''} 
    }



    [object] GetMaster([string] $stencilCaption, [string] $masterNameU)
    {
        $fullMstName = $stencilCaption, $masterNameU -join $this.stencilMasterDelimiter

        #Get 'local' master if it's already been used
        if ($this.masters.Contains($fullMstName))
        {
            return $this.masters[$fullMstName]
        }

        #otherwise try and retrieve the right stencil
        $tempStencil = $this.stencilWindows | Where-Object {$_.Caption -eq $stencilCaption} | Select-Object -First 1 | Select-Object -ExpandProperty Document      
        
        if ($tempStencil -eq $null)
        {
            Write-Host "Unable to find stencil with caption: '$stencilCaption'" -ForegroundColor Red
            return $null
        }
        else
        {
            $tempMst = $tempStencil.Masters | Where-Object {$_.NameU -eq $masterNameU} | Select-Object -First 1
            if ($tempMst -eq $null)
            {
                Write-Host "Unable to find master '$masterNameU' in stencil '$stencilCaption'" -ForegroundColor Red
                return $null
            }
            else
            {
                $this.masters.Add($fullMstName, $tempMst) 
                return $this.masters[$fullMstName]            
            }
        }
    }


    
    [object] DropShape([string] $stencilCaption, [string] $masterName)
    {
        return $this.DropShp($stencilCaption, $masterName, $false)    
    }

    [object] DropChildShape([string] $stencilCaption, [string] $masterName)
    {
        return $this.DropShp($stencilCaption, $masterName, $true)  
    }


    hidden [object] DropShp([string] $stencilCaption, [string] $masterName, [bool] $dropAsChild)
    {
        $mst = $this.GetMaster($stencilCaption, $masterName)
        if ($mst -ne $null)
        {
          if ($dropAsChild -eq $true)
          {
            if ($this.vCurrentParentShape -eq $null)
            {
                Write-Verbose "CurrentParentShape is null - Dropping '$masterName' shape unconnected"
                $newShp = $this.vCurrentPage.Drop($mst, 4.0, 7.0)
            }
            else
            {
              $newShp = $this.vCurrentPage.DropConnected($mst, $this.vCurrentParentShape, 2)            
            }
          }
          else
          {
            $newShp = $this.vCurrentPage.Drop($mst, 4.0, 7.0)
          }
          return $newShp
        }
        return $null    
    }



    [void] SetShapeData([object] $shpTarget, [hashtable] $shpData)
    {
        if ($shpTarget -ne $null)
        {
            $shpData.Keys | ForEach-Object {
                $rowName = "Prop.$_" 
                if ($shpTarget.CellExistsU($rowName, 0) -eq 0)
                {
                  $shpTarget.AddNamedRow(243, $_, 0)
                }
                $shpTarget.CellsU("$rowName.Value").FormulaU = "=$($shpData.Item($_))"
            }           
        }        
    }




    VisioTemplateDraw()
    {
        $this.Init('')
    }

    VisioTemplateDraw([string] $targetTemplateName)
    {
        $this.Init($targetTemplateName)
    }
    
} 









function Get-Webs
{
    [CmdletBinding()]
    Param
    (
        [Parameter(Mandatory=$true,
                   ValueFromPipeline=$true)]
        $web,

        [VisioTemplateDraw]
        $visi,

        [object]
        $parentShp,

        [string]
        $parentUrl,

        [string]
        $siteUrl
    )

    Process {
        $context = Get-SPOContext
        $context.Load($web)
        $context.Load($web.Webs)  
        $context.ExecuteQuery()

        $relativeUrl = ($web.Url) -replace $parentUrl, '' 
        
        $visi.vCurrentParentShape = $parentShp
        $webShp = $visi.DropChildShape('Sharepoint Online Stencil', 'Web')
        $visi.SetShapeData($webShp, @{Url=$(Add-Quotes $web.Url) 
                                      RelativeUrl=$(Add-Quotes $relativeUrl)
                                      SiteCollectionUrl=$(Add-Quotes $siteUrl)
                                      ParentUrl=$(Add-Quotes $parentUrl)
                                      WebTemplate=$(Add-Quotes $web.WebTemplate)}) 

        Write-Verbose "Dropped Web `($($web.Url)`)"
        
        ForEach ($subweb in $web.Webs) {  
            $subweb | Get-Webs -visi $visi -parentShp $webShp -parentUrl $web.Url -siteUrl $siteUrl
        }
     }
}


function Add-Quotes
{
    param
    (
        [string]
        $strIn
    )
    "`"$strIn`""
}



function New-OnlineTenantDiagram
{
     param
     (
         [string]
         $adminUrl,

         [string]
         $credname
     )   

    Connect-SPOnline -Url $adminUrl -Credential $credname
    if ($?)
    {
        $visiDraw = [VisioTemplateDraw]::new('SharePoint Online.vstx')
        if ($visiDraw.errorLoadingTemplate -ne $true)
        {
            $tenantShp = $visiDraw.DropShape('Sharepoint Online Stencil', 'Tenant')
            $visiDraw.SetShapeData($tenantShp, @{Url=$(Add-Quotes $adminUrl)})
    
            #Get SiteProperties but exclude public site
            $siteColProps = Get-SPOTenantSite -Detailed | Where-Object Url -notlike '*-public.sharepoint.com/' | Select-Object -Property Url
    
            ForEach ($siteColProp in $siteColProps){
                Connect-SPOnline -Url $siteColProp.Url -Credential $credname
        
                $site = Get-SPOSite

                Write-Verbose "$($site.Url) `(Site collection`)"

                #Drop Site Collection shape
                $visiDraw.vCurrentParentShape = $tenantShp
                $siteColShp = $visiDraw.DropChildShape('SharePoint Online Stencil', 'Site Collection')
                $visiDraw.SetShapeData($siteColShp, @{Url=$(Add-Quotes $site.Url)})       
        
                Get-SPOWeb | Get-Webs -visi $visiDraw -parentShp $siteColShp -parentUrl $site.Url -siteUrl $site.Url
            }

            $visiDraw.vDoc.DiagramServicesEnabled = 1 #visServiceAutoSizePage
            $visiDraw.vCurrentPage.Layout()
            $visiDraw.vCurrentPage.ResizeToFitContents()
            $visiDraw.vApp.ActiveWindow.Zoom = -1 #fit page to window
        }
    }
}







#  New-OnlineTenantDiagram -adminUrl https://[TENENT_NAME]-admin.sharepoint.com -credname [CREDENTIALS_NAME]






 